"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractFile = void 0;
var tslib_1 = require("tslib");
var sha256_uint8array_1 = require("sha256-uint8array");
var univ_conv_1 = require("univ-conv");
var AbstractDirectory_1 = require("./AbstractDirectory");
var AbstractEntry_1 = require("./AbstractEntry");
var errors_1 = require("./errors");
var util_1 = require("./util");
var AbstractFile = (function (_super) {
    (0, tslib_1.__extends)(AbstractFile, _super);
    function AbstractFile(fs, path) {
        var _a;
        var _this = _super.call(this, fs, path) || this;
        var hook = (_a = fs.options) === null || _a === void 0 ? void 0 : _a.hook;
        if (hook) {
            _this.beforeGet = hook.beforeGet;
            _this.beforePost = hook.beforePost;
            _this.beforePut = hook.beforePut;
        }
        return _this;
    }
    AbstractFile.prototype._convert = function (converter, chunk, type) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var _a, base64, binaryString, text;
            return (0, tslib_1.__generator)(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        _a = type;
                        switch (_a) {
                            case "ArrayBuffer": return [3, 1];
                            case "Uint8Array": return [3, 2];
                            case "Buffer": return [3, 3];
                            case "Blob": return [3, 4];
                            case "Base64": return [3, 5];
                            case "BinaryString": return [3, 7];
                            case "Text": return [3, 9];
                        }
                        return [3, 11];
                    case 1: return [2, converter.toArrayBuffer(chunk)];
                    case 2: return [2, converter.toUint8Array(chunk)];
                    case 3: return [2, converter.toBuffer(chunk)];
                    case 4: return [2, converter.toBlob(chunk)];
                    case 5: return [4, converter.toBase64(chunk)];
                    case 6:
                        base64 = _b.sent();
                        return [2, { value: base64, encoding: "Base64" }];
                    case 7: return [4, converter.toBinaryString(chunk)];
                    case 8:
                        binaryString = _b.sent();
                        return [2, { value: binaryString, encoding: "BinaryString" }];
                    case 9: return [4, converter.toBinaryString(chunk)];
                    case 10:
                        text = _b.sent();
                        return [2, { value: text, encoding: "Text" }];
                    case 11: return [2];
                }
            });
        });
    };
    AbstractFile.prototype._delete = function (options, _errors) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var stats, e_1;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 2, , 3]);
                        return [4, this.head()];
                    case 1:
                        stats = _a.sent();
                        if (stats.size == null) {
                            throw (0, errors_1.createError)({
                                name: errors_1.TypeMismatchError.name,
                                repository: this.fs.repository,
                                path: this.path,
                                e: "\"" + this.path + "\" is not a file",
                            });
                        }
                        return [3, 3];
                    case 2:
                        e_1 = _a.sent();
                        if (e_1.name === errors_1.NotFoundError.name) {
                            if (!options.force) {
                                throw e_1;
                            }
                        }
                        else {
                            throw (0, errors_1.createError)({
                                name: errors_1.NotReadableError.name,
                                repository: this.fs.repository,
                                path: this.path,
                                e: e_1,
                            });
                        }
                        return [3, 3];
                    case 3: return [2, this._rm()];
                }
            });
        });
    };
    AbstractFile.prototype._joinChunks = function (chunks, length, type) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var u8, ab, u8Pos, _i, chunks_1, chunk, u8Chunk, blobs, _a, chunks_2, chunk;
            return (0, tslib_1.__generator)(this, function (_b) {
                switch (type) {
                    case "ArrayBuffer":
                    case "Uint8Array":
                    case "Buffer":
                        u8 = void 0;
                        if (type === "Buffer") {
                            u8 = Buffer.alloc(length);
                        }
                        else {
                            ab = new ArrayBuffer(length);
                            u8 = new Uint8Array(ab);
                        }
                        u8Pos = 0;
                        for (_i = 0, chunks_1 = chunks; _i < chunks_1.length; _i++) {
                            chunk = chunks_1[_i];
                            u8Chunk = type === "ArrayBuffer"
                                ? new Uint8Array(chunk)
                                : chunk;
                            u8.set(u8Chunk, u8Pos);
                            u8Pos += u8Chunk.byteLength;
                        }
                        return [2, type === "ArrayBuffer" ? u8.buffer : u8];
                    case "Blob":
                        blobs = [];
                        for (_a = 0, chunks_2 = chunks; _a < chunks_2.length; _a++) {
                            chunk = chunks_2[_a];
                            blobs.push(chunk);
                        }
                        return [2, new Blob(blobs)];
                    case "Base64":
                    case "BinaryString":
                    case "Text":
                        return [2, { value: chunks.join(""), encoding: type }];
                }
                return [2];
            });
        });
    };
    AbstractFile.prototype._xmit = function (toEntry, _copyErrors, options) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var to, e_2, rs, create, e_3, ws;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (toEntry instanceof AbstractDirectory_1.AbstractDirectory) {
                            throw (0, errors_1.createError)({
                                name: errors_1.TypeMismatchError.name,
                                repository: toEntry.fs.repository,
                                path: toEntry.path,
                                e: "\"" + toEntry + "\" is not a file",
                            });
                        }
                        to = toEntry;
                        _a.label = 1;
                    case 1:
                        _a.trys.push([1, 3, , 4]);
                        return [4, to.head()];
                    case 2:
                        _a.sent();
                        if (!options.force) {
                            throw (0, errors_1.createError)({
                                name: errors_1.SecurityError.name,
                                repository: to.fs.repository,
                                path: to.path,
                            });
                        }
                        return [3, 4];
                    case 3:
                        e_2 = _a.sent();
                        if (e_2.name !== errors_1.NotFoundError.name) {
                            throw (0, errors_1.createError)({
                                name: errors_1.NotReadableError.name,
                                repository: to.fs.repository,
                                path: to.path,
                                e: e_2,
                            });
                        }
                        return [3, 4];
                    case 4: return [4, this.createReadStream({ bufferSize: options.bufferSize })];
                    case 5:
                        rs = _a.sent();
                        _a.label = 6;
                    case 6:
                        _a.trys.push([6, , 17, 19]);
                        create = void 0;
                        _a.label = 7;
                    case 7:
                        _a.trys.push([7, 9, , 10]);
                        return [4, to.head()];
                    case 8:
                        _a.sent();
                        create = false;
                        return [3, 10];
                    case 9:
                        e_3 = _a.sent();
                        if (e_3.name === errors_1.NotFoundError.name) {
                            create = true;
                        }
                        else {
                            throw (0, errors_1.createError)({
                                name: errors_1.NotReadableError.name,
                                repository: toEntry.fs.repository,
                                path: toEntry.path,
                                e: e_3,
                            });
                        }
                        return [3, 10];
                    case 10: return [4, to.createWriteStream({
                            append: false,
                            create: create,
                            bufferSize: options.bufferSize,
                        })];
                    case 11:
                        ws = _a.sent();
                        _a.label = 12;
                    case 12:
                        _a.trys.push([12, , 14, 16]);
                        return [4, rs.pipe(ws)];
                    case 13:
                        _a.sent();
                        return [3, 16];
                    case 14: return [4, ws.close()];
                    case 15:
                        _a.sent();
                        return [7];
                    case 16: return [3, 19];
                    case 17: return [4, rs.close()];
                    case 18:
                        _a.sent();
                        return [7];
                    case 19: return [2];
                }
            });
        });
    };
    AbstractFile.prototype.createReadStream = function (options) {
        if (options === void 0) { options = {}; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var rs;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (!(!options.ignoreHook && this.beforeGet)) return [3, 2];
                        return [4, this.beforeGet(this.path, options)];
                    case 1:
                        rs = _a.sent();
                        _a.label = 2;
                    case 2:
                        if (!!rs) return [3, 4];
                        return [4, this._createReadStream(options)];
                    case 3:
                        rs = _a.sent();
                        _a.label = 4;
                    case 4: return [2, rs];
                }
            });
        });
    };
    AbstractFile.prototype.createWriteStream = function (options) {
        if (options === void 0) { options = { append: false, create: true }; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var ws, stats, e_4;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        _a.trys.push([0, 4, , 9]);
                        return [4, this.head()];
                    case 1:
                        stats = _a.sent();
                        if (stats.size == null) {
                            throw (0, errors_1.createError)({
                                name: errors_1.TypeMismatchError.name,
                                repository: this.fs.repository,
                                path: this.path,
                                e: "\"" + this.path + "\" is directory",
                            });
                        }
                        if (options.create) {
                            throw (0, errors_1.createError)({
                                name: errors_1.SecurityError.name,
                                repository: this.fs.repository,
                                path: this.path,
                                e: "\"" + this.path + "\" has already exists",
                            });
                        }
                        if (!(!options.ignoreHook && this.beforePut)) return [3, 3];
                        return [4, this.beforePut(this.path, options)];
                    case 2:
                        ws = _a.sent();
                        _a.label = 3;
                    case 3: return [3, 9];
                    case 4:
                        e_4 = _a.sent();
                        if (!(e_4.name === errors_1.NotFoundError.name)) return [3, 7];
                        if (!(!options.ignoreHook && this.beforePost)) return [3, 6];
                        return [4, this.beforePost(this.path, options)];
                    case 5:
                        ws = _a.sent();
                        _a.label = 6;
                    case 6: return [3, 8];
                    case 7: throw (0, errors_1.createError)({
                        name: errors_1.NotReadableError.name,
                        repository: this.fs.repository,
                        path: this.path,
                        e: e_4,
                    });
                    case 8:
                        options.create = true;
                        return [3, 9];
                    case 9:
                        if (!!ws) return [3, 11];
                        return [4, this._createWriteStream(options)];
                    case 10:
                        ws = _a.sent();
                        _a.label = 11;
                    case 11: return [2, ws];
                }
            });
        });
    };
    AbstractFile.prototype.hash = function (options) {
        if (options === void 0) { options = {}; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var rs, c, hash, result, buffer;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.createReadStream(options)];
                    case 1:
                        rs = _a.sent();
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, , 7, 9]);
                        c = new univ_conv_1.Converter({ bufferSize: options.bufferSize });
                        hash = (0, sha256_uint8array_1.createHash)();
                        result = void 0;
                        _a.label = 3;
                    case 3: return [4, rs.read()];
                    case 4:
                        if (!((result = _a.sent()) != null)) return [3, 6];
                        return [4, c.toUint8Array(result)];
                    case 5:
                        buffer = _a.sent();
                        hash.update(buffer);
                        return [3, 3];
                    case 6: return [2, (0, util_1.toHex)(hash.digest())];
                    case 7: return [4, rs.close()];
                    case 8:
                        _a.sent();
                        return [7];
                    case 9: return [2];
                }
            });
        });
    };
    AbstractFile.prototype.readAll = function (options) {
        if (options === void 0) { options = { sourceType: "Uint8Array" }; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var rs, type, converter, pos, chunks, chunk, converted;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        (0, univ_conv_1.validateBufferSize)(options);
                        return [4, this.createReadStream(options)];
                    case 1:
                        rs = (_a.sent());
                        type = options.sourceType;
                        converter = rs.converter;
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, , 7, 9]);
                        pos = 0;
                        chunks = [];
                        chunk = void 0;
                        _a.label = 3;
                    case 3: return [4, rs.read()];
                    case 4:
                        if (!((chunk = _a.sent()) != null)) return [3, 6];
                        pos += (0, univ_conv_1.getSize)(chunk);
                        return [4, this._convert(converter, chunk, type)];
                    case 5:
                        converted = _a.sent();
                        chunks.push(converted);
                        return [3, 3];
                    case 6: return [2, this._joinChunks(chunks, pos, type)];
                    case 7: return [4, rs.close()];
                    case 8:
                        _a.sent();
                        return [7];
                    case 9: return [2];
                }
            });
        });
    };
    AbstractFile.prototype.writeAll = function (src, options) {
        if (options === void 0) { options = { append: false, create: true }; }
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var bufferSize, ws, pos, chunk, u8, pos, chunk;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        bufferSize = (0, univ_conv_1.validateBufferSize)(options);
                        return [4, this.createWriteStream(options)];
                    case 1:
                        ws = (_a.sent());
                        if (!(0, univ_conv_1.isBlob)(src)) return [3, 10];
                        _a.label = 2;
                    case 2:
                        _a.trys.push([2, , 7, 9]);
                        pos = 0;
                        chunk = void 0;
                        _a.label = 3;
                    case 3:
                        chunk = src.slice(pos, pos + bufferSize);
                        pos += bufferSize;
                        if (!(0 < chunk.size)) return [3, 5];
                        return [4, ws.write(chunk)];
                    case 4:
                        _a.sent();
                        _a.label = 5;
                    case 5:
                        if (chunk.size === bufferSize) return [3, 3];
                        _a.label = 6;
                    case 6: return [3, 9];
                    case 7: return [4, ws.close()];
                    case 8:
                        _a.sent();
                        return [7];
                    case 9: return [2, src.size];
                    case 10: return [4, ws.converter.toUint8Array(src)];
                    case 11:
                        u8 = _a.sent();
                        _a.label = 12;
                    case 12:
                        _a.trys.push([12, , 17, 19]);
                        pos = 0;
                        chunk = void 0;
                        _a.label = 13;
                    case 13:
                        chunk = u8.subarray(pos, pos + bufferSize);
                        pos += bufferSize;
                        if (!(0 < chunk.byteLength)) return [3, 15];
                        return [4, ws.write(chunk)];
                    case 14:
                        _a.sent();
                        _a.label = 15;
                    case 15:
                        if (chunk.byteLength === bufferSize) return [3, 13];
                        _a.label = 16;
                    case 16: return [3, 19];
                    case 17: return [4, ws.close()];
                    case 18:
                        _a.sent();
                        return [7];
                    case 19: return [2, u8.byteLength];
                }
            });
        });
    };
    AbstractFile.prototype._createBuffer = function (byteLength) {
        var ab = new ArrayBuffer(byteLength);
        return new Uint8Array(ab);
    };
    return AbstractFile;
}(AbstractEntry_1.AbstractEntry));
exports.AbstractFile = AbstractFile;
//# sourceMappingURL=AbstractFile.js.map