"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.AbstractReadStream = void 0;
var tslib_1 = require("tslib");
var univ_conv_1 = require("univ-conv");
var AbstractStream_1 = require("./AbstractStream");
var AbstractReadStream = (function (_super) {
    (0, tslib_1.__extends)(AbstractReadStream, _super);
    function AbstractReadStream(file, options) {
        var _a;
        var _this = _super.call(this, file, options) || this;
        _this.options = options;
        _this.handled = false;
        if (!options.sourceType) {
            options.sourceType = _this.getDefaultSourceType();
        }
        _this.afterGet = (_a = file.fs.options.hook) === null || _a === void 0 ? void 0 : _a.afterGet;
        return _this;
    }
    AbstractReadStream.prototype.close = function () {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this._close()];
                    case 1:
                        _a.sent();
                        this.position = 0;
                        if (!this.options.ignoreHook && this.afterGet) {
                            this.afterGet(this.file.path);
                        }
                        return [2];
                }
            });
        });
    };
    AbstractReadStream.prototype.pipe = function (ws) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var buffer;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0: return [4, this.read()];
                    case 1:
                        if (!((buffer = _a.sent()) != null)) return [3, 3];
                        return [4, ws.write(buffer)];
                    case 2:
                        _a.sent();
                        return [3, 0];
                    case 3: return [2];
                }
            });
        });
    };
    AbstractReadStream.prototype.read = function (size) {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var file, converter, souceType, result, pos, chunk, chunks, fileSize, max, next, chunkSize, chunk, converted;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        file = this.file;
                        converter = this.converter;
                        souceType = this.options.sourceType;
                        result = null;
                        pos = 0;
                        if (!(size == null || size <= this.bufferSize)) return [3, 4];
                        return [4, this._read(size)];
                    case 1:
                        chunk = _a.sent();
                        if (!chunk) return [3, 3];
                        pos = (0, univ_conv_1.getSize)(chunk);
                        return [4, file._convert(converter, chunk, souceType)];
                    case 2:
                        result = _a.sent();
                        _a.label = 3;
                    case 3: return [3, 12];
                    case 4:
                        chunks = [];
                        return [4, this._getFileSize()];
                    case 5:
                        fileSize = _a.sent();
                        max = Math.min(size, fileSize);
                        _a.label = 6;
                    case 6:
                        next = pos + this.bufferSize;
                        if (max < next) {
                            next = max;
                        }
                        chunkSize = next - pos;
                        return [4, this._read(chunkSize)];
                    case 7:
                        chunk = _a.sent();
                        if (!chunk) {
                            return [3, 10];
                        }
                        pos += (0, univ_conv_1.getSize)(chunk);
                        return [4, file._convert(converter, chunk, souceType)];
                    case 8:
                        converted = _a.sent();
                        chunks.push(converted);
                        _a.label = 9;
                    case 9:
                        if (pos < max) return [3, 6];
                        _a.label = 10;
                    case 10:
                        if (!(0 < chunks.length)) return [3, 12];
                        return [4, file._joinChunks(chunks, pos, souceType)];
                    case 11:
                        result = _a.sent();
                        _a.label = 12;
                    case 12:
                        this.position += pos;
                        this.handled = true;
                        return [2, result];
                }
            });
        });
    };
    AbstractReadStream.prototype._getFileSize = function () {
        return (0, tslib_1.__awaiter)(this, void 0, void 0, function () {
            var stats, fileSize;
            return (0, tslib_1.__generator)(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        if (this.fileSize) {
                            return [2, this.fileSize];
                        }
                        return [4, this.file.head({ ignoreHook: true })];
                    case 1:
                        stats = _a.sent();
                        fileSize = stats.size;
                        this.fileSize = fileSize;
                        return [2, fileSize];
                }
            });
        });
    };
    return AbstractReadStream;
}(AbstractStream_1.AbstractStream));
exports.AbstractReadStream = AbstractReadStream;
//# sourceMappingURL=AbstractReadStream.js.map