"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.testAll = void 0;
var tslib_1 = require("tslib");
var core_1 = require("../core");
var errors_1 = require("../errors");
var univ_conv_1 = require("univ-conv");
var c = new univ_conv_1.Converter();
var testAll = function (fs) {
    test("rootdir", function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var dir, paths;
        return (0, tslib_1.__generator)(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, fs.getDirectory("/")];
                case 1:
                    dir = _a.sent();
                    return [4, dir.readdir()];
                case 2:
                    paths = _a.sent();
                    expect(paths.length).toBe(0);
                    return [2];
            }
        });
    }); });
    test("add empty file", function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var file, e_1, buffer, ws, stats;
        return (0, tslib_1.__generator)(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, fs.getFile("/empty.txt")];
                case 1:
                    file = _a.sent();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4, file.stat()];
                case 3:
                    _a.sent();
                    fail("Found file: " + file.path);
                    return [3, 5];
                case 4:
                    e_1 = _a.sent();
                    expect(e_1.name).toBe(errors_1.NotFoundError.name);
                    return [3, 5];
                case 5: return [4, c.toArrayBuffer({ value: "", encoding: "Text" })];
                case 6:
                    buffer = _a.sent();
                    return [4, file.createWriteStream()];
                case 7:
                    ws = _a.sent();
                    return [4, ws.write(buffer)];
                case 8:
                    _a.sent();
                    return [4, ws.close()];
                case 9:
                    _a.sent();
                    return [4, file.stat()];
                case 10:
                    stats = _a.sent();
                    expect(stats.size).toBe(0);
                    return [2];
            }
        });
    }); });
    test("add text file", function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var file, e_2, buffer, ws, stats;
        return (0, tslib_1.__generator)(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, fs.getFile("/test.txt")];
                case 1:
                    file = _a.sent();
                    _a.label = 2;
                case 2:
                    _a.trys.push([2, 4, , 5]);
                    return [4, file.stat()];
                case 3:
                    _a.sent();
                    fail("Found file: " + file.path);
                    return [3, 5];
                case 4:
                    e_2 = _a.sent();
                    expect(e_2.name).toBe(errors_1.NotFoundError.name);
                    return [3, 5];
                case 5: return [4, c.toArrayBuffer({ value: "test", encoding: "Text" })];
                case 6:
                    buffer = _a.sent();
                    return [4, file.createWriteStream()];
                case 7:
                    ws = _a.sent();
                    return [4, ws.write(buffer)];
                case 8:
                    _a.sent();
                    return [4, ws.close()];
                case 9:
                    _a.sent();
                    return [4, file.stat()];
                case 10:
                    stats = _a.sent();
                    expect(stats.size).toBe(4);
                    return [2];
            }
        });
    }); });
    test("read text file", function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var file, rs, buffer, text;
        return (0, tslib_1.__generator)(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, fs.getFile("/test.txt")];
                case 1:
                    file = _a.sent();
                    return [4, file.createReadStream()];
                case 2:
                    rs = _a.sent();
                    return [4, rs.read()];
                case 3:
                    buffer = (_a.sent());
                    expect(buffer.byteLength).toBe(4);
                    return [4, c.toText(buffer)];
                case 4:
                    text = _a.sent();
                    expect(text).toBe("test");
                    return [2];
            }
        });
    }); });
    test("continuous read and write", function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var file, ws, _a, _b, _c, _d, rs, buffer, text, _e, _f;
        return (0, tslib_1.__generator)(this, function (_g) {
            switch (_g.label) {
                case 0: return [4, fs.getFile("/otani.txt")];
                case 1:
                    file = _g.sent();
                    return [4, file.createWriteStream()];
                case 2:
                    ws = _g.sent();
                    _b = (_a = ws).write;
                    return [4, c.toArrayBuffer({ value: "大谷", encoding: "Text" })];
                case 3: return [4, _b.apply(_a, [_g.sent()])];
                case 4:
                    _g.sent();
                    _d = (_c = ws).write;
                    return [4, c.toArrayBuffer({ value: "翔平", encoding: "Text" })];
                case 5: return [4, _d.apply(_c, [_g.sent()])];
                case 6:
                    _g.sent();
                    return [4, file.createReadStream()];
                case 7:
                    rs = _g.sent();
                    return [4, rs.read(6)];
                case 8:
                    buffer = (_g.sent());
                    return [4, c.toText(buffer)];
                case 9:
                    text = _g.sent();
                    expect(text).toBe("大谷");
                    return [4, rs.seek(6, core_1.SeekOrigin.Begin)];
                case 10:
                    _g.sent();
                    return [4, rs.read()];
                case 11:
                    buffer = (_g.sent());
                    return [4, c.toText(buffer)];
                case 12:
                    text = _g.sent();
                    expect(text).toBe("翔平");
                    return [4, ws.seek(0, core_1.SeekOrigin.End)];
                case 13:
                    _g.sent();
                    _f = (_e = ws).write;
                    return [4, c.toArrayBuffer({ value: "ホームラン", encoding: "Text" })];
                case 14: return [4, _f.apply(_e, [_g.sent()])];
                case 15:
                    _g.sent();
                    return [4, rs.seek(0, core_1.SeekOrigin.Begin)];
                case 16:
                    _g.sent();
                    return [4, rs.read()];
                case 17:
                    buffer = (_g.sent());
                    return [4, c.toText(buffer)];
                case 18:
                    text = _g.sent();
                    expect(text).toBe("大谷翔平ホームラン");
                    return [4, rs.seek(0, core_1.SeekOrigin.Begin)];
                case 19:
                    _g.sent();
                    return [4, rs.read(6)];
                case 20:
                    _g.sent();
                    return [4, rs.seek(6, core_1.SeekOrigin.Current)];
                case 21:
                    _g.sent();
                    return [4, rs.read()];
                case 22:
                    buffer = (_g.sent());
                    return [4, c.toText(buffer)];
                case 23:
                    text = _g.sent();
                    expect(text).toBe("ホームラン");
                    return [4, ws.close()];
                case 24:
                    _g.sent();
                    return [4, rs.close()];
                case 25:
                    _g.sent();
                    return [2];
            }
        });
    }); });
    test("mkdir test", function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var dir, dirs, folder, e_3;
        return (0, tslib_1.__generator)(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, fs.getDirectory("/")];
                case 1:
                    dir = _a.sent();
                    return [4, dir.readdir()];
                case 2:
                    dirs = _a.sent();
                    expect(dirs.length).toBe(3);
                    expect(0 <= dirs.indexOf("/empty.txt")).toBe(true);
                    expect(0 <= dirs.indexOf("/test.txt")).toBe(true);
                    expect(0 <= dirs.indexOf("/otani.txt")).toBe(true);
                    return [4, fs.getDirectory("/folder")];
                case 3:
                    folder = _a.sent();
                    _a.label = 4;
                case 4:
                    _a.trys.push([4, 6, , 7]);
                    return [4, folder.stat()];
                case 5:
                    _a.sent();
                    fail("Found folder: " + folder.path);
                    return [3, 7];
                case 6:
                    e_3 = _a.sent();
                    expect(e_3.name).toBe(errors_1.NotFoundError.name);
                    return [3, 7];
                case 7: return [4, folder.mkdir()];
                case 8:
                    _a.sent();
                    return [4, folder.stat()];
                case 9:
                    _a.sent();
                    return [4, dir.readdir()];
                case 10:
                    dirs = _a.sent();
                    expect(dirs.length).toBe(4);
                    expect(0 <= dirs.indexOf("/empty.txt")).toBe(true);
                    expect(0 <= dirs.indexOf("/test.txt")).toBe(true);
                    expect(0 <= dirs.indexOf("/otani.txt")).toBe(true);
                    expect(0 <= dirs.indexOf("/folder")).toBe(true);
                    return [2];
            }
        });
    }); });
    test("create file in dir", function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var file, e_4, ws, outBuf, before, after, stats, modified, rs, inBuf, text, dir, list;
        var _a;
        return (0, tslib_1.__generator)(this, function (_b) {
            switch (_b.label) {
                case 0: return [4, fs.getFile("/folder/sample.txt")];
                case 1:
                    file = _b.sent();
                    _b.label = 2;
                case 2:
                    _b.trys.push([2, 4, , 5]);
                    return [4, file.stat()];
                case 3:
                    _b.sent();
                    fail("Found file: " + file.path);
                    return [3, 5];
                case 4:
                    e_4 = _b.sent();
                    expect(e_4.name).toBe(errors_1.NotFoundError.name);
                    return [3, 5];
                case 5: return [4, file.createWriteStream()];
                case 6:
                    ws = _b.sent();
                    return [4, c.toArrayBuffer({ value: "Sample", encoding: "Text" })];
                case 7:
                    outBuf = _b.sent();
                    before = Date.now();
                    return [4, ws.write(outBuf)];
                case 8:
                    _b.sent();
                    return [4, ws.close()];
                case 9:
                    _b.sent();
                    after = Date.now() + 1;
                    return [4, file.stat()];
                case 10:
                    stats = _b.sent();
                    modified = (_a = stats.modified) !== null && _a !== void 0 ? _a : 0;
                    expect(before <= modified && modified <= after).toBe(true);
                    return [4, file.createReadStream()];
                case 11:
                    rs = _b.sent();
                    return [4, rs.read()];
                case 12:
                    inBuf = (_b.sent());
                    return [4, c.toText(inBuf)];
                case 13:
                    text = _b.sent();
                    expect(text).toBe("Sample");
                    rs.close();
                    return [4, fs.getDirectory("/folder/")];
                case 14:
                    dir = _b.sent();
                    return [4, dir.list()];
                case 15:
                    list = _b.sent();
                    expect(0 <= list.indexOf("/folder/sample.txt")).toBe(true);
                    return [2];
            }
        });
    }); });
    test("copy directory", function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var from, to, errors, stats, root, list, toList;
        return (0, tslib_1.__generator)(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, fs.getDirectory("/folder")];
                case 1:
                    from = _a.sent();
                    return [4, fs.getDirectory("/folder2")];
                case 2:
                    to = _a.sent();
                    return [4, from.copy(to, { force: false, recursive: true })];
                case 3:
                    errors = _a.sent();
                    expect(errors.length).toBe(0);
                    return [4, to.stat()];
                case 4:
                    stats = _a.sent();
                    expect(stats.size).toBeUndefined();
                    return [4, fs.getDirectory("/")];
                case 5:
                    root = _a.sent();
                    return [4, root.ls()];
                case 6:
                    list = _a.sent();
                    expect(0 <= list.indexOf("/folder2")).toBe(true);
                    return [4, to.ls()];
                case 7:
                    toList = _a.sent();
                    expect(0 <= toList.indexOf("/folder2/sample.txt")).toBe(true);
                    return [2];
            }
        });
    }); });
    test("move file", function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var list;
        return (0, tslib_1.__generator)(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, fs.move("/folder2/sample.txt", "/folder2/sample2.txt")];
                case 1:
                    _a.sent();
                    return [4, fs.list("/folder2")];
                case 2:
                    list = _a.sent();
                    expect(list.indexOf("/folder2/sample.txt") < 0).toBe(true);
                    expect(0 <= list.indexOf("/folder2/sample2.txt")).toBe(true);
                    return [2];
            }
        });
    }); });
    test("move directory", function () { return (0, tslib_1.__awaiter)(void 0, void 0, void 0, function () {
        var errors, root, list, folder3, folder3List;
        return (0, tslib_1.__generator)(this, function (_a) {
            switch (_a.label) {
                case 0: return [4, fs.move("/folder2", "/folder3")];
                case 1:
                    errors = _a.sent();
                    expect(errors.length).toBe(0);
                    return [4, fs.getDirectory("/")];
                case 2:
                    root = _a.sent();
                    return [4, root.ls()];
                case 3:
                    list = _a.sent();
                    expect(list.indexOf("/folder2") < 0).toBe(true);
                    expect(0 <= list.indexOf("/folder3")).toBe(true);
                    return [4, fs.getDirectory("/folder3")];
                case 4:
                    folder3 = _a.sent();
                    return [4, folder3.ls()];
                case 5:
                    folder3List = _a.sent();
                    expect(0 <= folder3List.indexOf("/folder3/sample2.txt")).toBe(true);
                    return [2];
            }
        });
    }); });
};
exports.testAll = testAll;
//# sourceMappingURL=basic.js.map